package recovery_worker

import (
	"context"
	"encoding/json"
	"gitea.pena/PenaSide/codeword/internal/adapters/client"
	"gitea.pena/PenaSide/codeword/internal/models"
	"github.com/go-redis/redis/v8"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.uber.org/zap"
	"time"
)

type Deps struct {
	Logger      *zap.Logger
	Redis       *redis.Client
	EmailSender *client.RecoveryEmailSender
	Mongo       *mongo.Collection
}

type RecoveryWorker struct {
	logger      *zap.Logger
	redis       *redis.Client
	emailSender *client.RecoveryEmailSender
	mongo       *mongo.Collection
}

func NewRecoveryWC(deps Deps) *RecoveryWorker {
	return &RecoveryWorker{
		logger:      deps.Logger,
		redis:       deps.Redis,
		emailSender: deps.EmailSender,
		mongo:       deps.Mongo,
	}
}

func (wc *RecoveryWorker) Start(ctx context.Context) {
	ticker := time.NewTicker(10 * time.Second)
	defer ticker.Stop()

	for {
		select {
		case <-ticker.C:
			wc.processTasks(ctx)
		case <-ctx.Done():
			return
		}
	}
}

func (wc *RecoveryWorker) processTasks(ctx context.Context) {
	var cursor uint64
	for {
		var keys []string
		var err error
		keys, cursor, err = wc.redis.Scan(ctx, cursor, "email:task:*", 0).Result()
		if err != nil {
			wc.logger.Error("Failed to scan for email tasks", zap.Error(err))
			break
		}

		for _, key := range keys {
			taskBytes, err := wc.redis.GetDel(ctx, key).Result()
			if err == redis.Nil {
				continue
			} else if err != nil {
				wc.logger.Error("Failed to getdel recovery task", zap.String("key", key), zap.Error(err))
				continue
			}

			var task models.RecoveryRecord
			if json.Unmarshal([]byte(taskBytes), &task) != nil {
				wc.logger.Error("Failed to unmarshal recovery task", zap.String("key", key), zap.String("task", taskBytes))
				continue
			}

			err = wc.sendRecoveryTask(ctx, task)
			if err != nil {
				wc.logger.Error("Failed to send recovery task", zap.String("key", key), zap.Error(err))
			}
		}
		if cursor == 0 {
			break
		}
	}
}

func (wc *RecoveryWorker) sendRecoveryTask(ctx context.Context, task models.RecoveryRecord) error {
	err := wc.emailSender.SendRecoveryEmail(task.Email, task.Key)
	if err != nil {
		wc.logger.Error("Failed to send recovery email", zap.Error(err))
		return err
	}

	update := bson.M{
		"$set": bson.M{
			"sent":    true,
			"sent_at": time.Now(),
		},
	}

	objectID, err := primitive.ObjectIDFromHex(task.ID)
	if err != nil {
		wc.logger.Error("Invalid ObjectID", zap.String("ID", task.ID), zap.Error(err))
		return err
	}

	filter := bson.M{"_id": objectID}

	result, err := wc.mongo.UpdateOne(ctx, filter, update)
	if err != nil {
		wc.logger.Error("Failed to update restore request", zap.Error(err))
		return err
	}

	if result.ModifiedCount == 0 {
		wc.logger.Warn("No documents were updated - this may indicate the document was not found",
			zap.String("ID", task.ID))
	}

	//wc.logger.Info("Recovery email sent and restore request updated successfully", zap.String("email", task.Email))
	return nil
}

func (wc *RecoveryWorker) Stop(ctx context.Context) error {
	return nil
}
